from django.db import models
from django import forms

import string
import random

def createRandomId():
    '''Return a random string of length 10.'''
    random.seed()
    d = [random.choice(string.letters) for x in range(10)]
    s = ''.join(d)
    return s

class UserSubmission(models.Model):
    """A model of a submission via the CellPublisher web interface. """
    
    # Random key for the diagram's administration interface
    user_admin_id = models.CharField(max_length=10,
                                     default=createRandomId,
                                     editable=False)

    # Random key for the URL of the resulting diagram
    url_id = models.CharField(max_length=10,
                              default=createRandomId,
                              editable=False)

    # Copyright owner of the SBML file
    owner = models.CharField(max_length=100)

    # Title of the resulting HTML page
    html_title = models.CharField(max_length=200)

    # Time when the job was submitted
    timestamp = models.DateTimeField(auto_now_add=True)

    # IP Address of the submitter
    ip = models.IPAddressField(editable=False)

    # SBML file to be parsed
    sbml = models.FileField(upload_to='%Y%m%d%H%M%S/')

    # PNG image to be processed
    png = models.ImageField(upload_to='%Y%m%d%H%M%S/')

    # Google maps API key
    maps_api_key = models.CharField(max_length=100, blank=True)

    # Permission to list the pathway in the index
    ok_upload = models.BooleanField(default=False)

    # E-Mail address to send the link to the finished pathway
    email = models.EmailField(blank=True)

    def __unicode__(self):
        return "'%s' created by %s" % (self.html_title, self.owner)

    def get_absolute_url(self):
        return "/view/%s/" % self.url_id

    def get_admin_url(self):
        return "/status/%s/" % self.user_admin_id

class UserSubmissionForm(forms.ModelForm):
    """
    A form for the submission based one-to-one on the UserSubmission model.

    """
    ok_upload = forms.BooleanField(required=False)
    class Meta:
        model = UserSubmission
        

class FinishedJob(models.Model):
    """A model of a job that was already processed."""
    # Submission to which this job refers to
    submission = models.OneToOneField(UserSubmission)

    # Time when the job was finished
    timestamp = models.DateTimeField(auto_now_add=True)

    # Status of the result. True for no errors or exceptions
    # False in the contrary case
    ok = models.BooleanField()

    # An indicator whether the pathway was deleted or not
    deleted = models.NullBooleanField(default=False)

    # MaxZoom of the pathway (if the creation was successful
    maxZoom = models.IntegerField(default=0)

    def __unicode__(self):
        return "Job corresponding to submission %d" % self.submission.pk
